from flask import Flask, request

from .interaction import interaction

app = Flask(__name__)


@app.route("/", methods=["POST"])
async def interactions_endpoint():
    raw_request = request.json
    signature = request.headers.get('X-Signature-Ed25519')
    timestamp = request.headers.get('X-Signature-Timestamp')
    print(f"👉 Request: {raw_request}")
    return interaction.interact(raw_request, signature, timestamp).as_server_response()


def start_server():
    app.run(host='0.0.0.0', debug=True, port=8123, threaded=True)



import json
from dataclasses import dataclass
from http.client import OK, UNAUTHORIZED


@dataclass
class HttpResponse:
    status_code: int
    body: [dict, str]
    headers: dict

    def __init__(self, body, *, headers=None):
        self.body = body
        self.headers = headers

    def as_serverless_response(self):
        return {"statusCode": self.status_code,
                "body": json.dumps(self.body) if isinstance(self.body, dict) else self.body,
                "headers": self.headers}

    def as_server_response(self):
        return self.body, self.status_code


class HttpOk(HttpResponse):
    status_code = OK


class HttpUnauthorized(HttpResponse):
    status_code = UNAUTHORIZED



import json

from flask import jsonify
from discord_interactions import verify_key

from .env import DISCORD_PUBLIC_KEY
from .types.api import HttpResponse, HttpUnauthorized, HttpOk


class Interaction:
    def __init__(self):
        self.commands = {}

    def interact(self, raw_request, signature, timestamp, client_public_key=DISCORD_PUBLIC_KEY) -> HttpResponse:
        if signature is None or timestamp is None or not verify_key(json.dumps(raw_request, separators=(',', ':'))
                                                                        .encode('utf-8'), signature, timestamp,
                                                                    client_public_key):
            return HttpUnauthorized('Bad request signature')

        if raw_request["type"] == 1:  # PING
            response_data = {"type": 1}  # PONG
        else:
            data = raw_request["data"]
            command_name = data["name"]

            message_content = self.commands[command_name]()

            if command_name == "echo":
                original_message = data["options"][0]["value"]
                message_content = f"Echoing: {original_message}"

            response_data = {
                "type": 4,
                "data": {"content": message_content},
            }

        return HttpOk(response_data, headers={"Content-Type": "application/json"})

    def command(self, *, name):
        def decorator(func):
            self.commands[name] = func
            return func

        return decorator

interaction = Interaction()



from dotenv import load_dotenv

from dislord import interaction, server

load_dotenv()


@interaction.command(name="hello")
def hello():
    return "hello world"


if __name__ == '__main__':
    server.start_server()



import json

from .interaction import interaction


def serverless_handler(event, context):
    if event['httpMethod'] == "POST":
        print(f"🫱 Full Event: {event}")
        raw_request = json.loads(event["body"])
        print(f"👉 Request: {raw_request}")
        raw_headers = event["headers"]
        signature = raw_headers.get('x-signature-ed25519')
        timestamp = raw_headers.get('x-signature-timestamp')
        return interaction.interact(raw_request, signature, timestamp).as_serverless_response()
